package adapter_test

import (
	"context"
	"testing"
	"time"

	"github.com/cenkalti/backoff/v4"
	"github.com/redis/go-redis/v9"
	"github.com/stretchr/testify/require"
)

func newRedisClient(tb testing.TB) *redis.Client {
	tb.Helper()

	opts, err := redis.ParseURL(redisConnectionString)
	require.NoError(tb, err, "couldn't parse redis connection string")

	client := redis.NewClient(opts)
	tb.Cleanup(func() {
		_ = client.Close()
	})

	bo := backoff.NewExponentialBackOff()
	bo.MaxInterval = time.Second * 5
	bo.MaxElapsedTime = 30 * time.Second
	require.NoError(tb, backoff.Retry(func() error {
		return client.Ping(context.Background()).Err()
	}, bo), "couldn't ping DB")

	return client
}
